program BINDconf;

{$APPTYPE CONSOLE}

uses
  WinSvc,
  WinInet,
  Windows,
  SysUtils,
  ShellAPI,
  Registry,
  FileCtrl;

function ConfigureBIND : Boolean; forward;
function DownloadFile(const AURL : String; const AFileName : String) : Boolean; forward;
function ExecuteProgram(const AFileName : String; const AParameters : String) : Boolean; forward;
function ServiceGetStatus(const AServiceName : String) : LongInt; forward;
function ServiceStart(const AServiceName : String) : Boolean; forward;
function ServiceStop(const AServiceName : String) : Boolean; forward;
function StripBackslash(const APath : String) : String; forward;

function ConfigureBIND : Boolean;
const
  CBackslash  = '\';
  CBinPath    = 'bin\';
  CConfGen    = 'rndc-confgen.exe';
  CEtcPath    = 'etc\';
  CKeyName    = 'rndc-key';
  CLocalZone  = 'localhost.zone';
  CNamed      = 'named';
  CNamedConf  = 'named.conf';
  CNamedLocal = 'named.local';
  CNamedPid   = 'named.pid';
  CNamedRoot  = 'named.root';
  CPortNum    = '953';
  CRegPath    = '\Software\ISC\BIND\';
  CRegValue   = 'InstallDir';
//  CRndc       = 'rndc.exe';
  CRndcConf   = 'rndc.conf';
  CRndcKey    = 'rndc.key';
  CRootURL    = 'http://www.internic.net/zones/named.root';
  CServerIP   = '127.0.0.1';
var
  LBindPath  : String;
  LIndex     : LongInt;
  LKeyFile   : array of String;
  LNextLine  : String;
  LReadFile  : TextFile;
  LRegistry  : TRegistry;
  LStartBind : Boolean;
  LStopped   : Boolean;
  LWriteFile : TextFile;
begin
  Result := false;

  LBindPath := '';

  LRegistry := TRegistry.Create(KEY_READ);
  try
    LRegistry.RootKey := HKEY_LOCAL_MACHINE;

    if LRegistry.OpenKey(CRegPath, false) then
    begin
      try
        LBindPath := LRegistry.ReadString(CRegValue);
        if (LBindPath[Length(LBindPath)] <> CBackslash) then
          LBindPath := LBindPath + CBackslash;
      finally
        LRegistry.CloseKey;
      end;
    end;
  finally
    LRegistry.Free;
  end;

  if DirectoryExists(LBindPath) then
  begin
    if DirectoryExists(LBindPath + CBinPath) then
    begin
//      if FileExists(LBindPath + CBinPath + CRndc) then
//      begin
        if FileExists(LBindPath + CBinPath + CConfGen) then
        begin
          if ForceDirectories(LBindPath + CEtcPath) then
          begin
            LStartBind := (ServiceGetStatus(CNamed) = SERVICE_RUNNING);

            LStopped := not(LStartBind);
            if not(LStopped) then
            begin
              WriteLn('Stopping service ...');
              LStopped := ServiceStop(CNamed);
            end;

            if LStopped then
            begin
              if FileExists(LBindPath + CEtcPath + CNamedPid) then
              begin
                WriteLn('Deleting named.pid ...');
                DeleteFile(LBindPath + CEtcPath + CNamedPid);
              end;

              if not(FileExists(LBindPath + CEtcPath + CNamedPid)) then
              begin
                WriteLn('Downloading named.root ...');
                if DownloadFile(CRootURL, LBindPath + CEtcPath + CNamedRoot) then
                begin
                  WriteLn('Deleting rndc.key ...');
                  DeleteFile(LBindPath + CEtcPath + CRndcKey);
                  
                  if not(FileExists(LBindPath + CEtcPath + CRndcKey)) then
                  begin
                    WriteLn('Creating rndc.key ...');
                    ExecuteProgram(LBindPath + CBinPath + CConfGen,
                                   '-a ' +
                                   '-b "512" ' +
                                   '-c "' + LBindPath + CEtcPath + CRndcKey + '" ' +
                                   '-k "' + CKeyName + '" ' +
                                   '-p "' + CPortNum + '" ' +
                                   '-s "' + CServerIP + '"');

                    if FileExists(LBindPath + CEtcPath + CRndcKey) then
                    begin
                      SetLength(LKeyFile, 0);

                      WriteLn('Reading rndc.key ...');
                      AssignFile(LReadFile, LBindPath + CEtcPath + CRndcKey);
                      Reset(LReadFile);
                      try
                        while not(EoF(LReadFile)) do
                        begin
                          ReadLn(LReadFile, LNextLine);

                          SetLength(LKeyFile, Succ(Length(LKeyFile)));
                          LKeyFile[High(LKeyFile)] := LNextLine
                        end;
                      finally
                        CloseFile(LReadFile);
                      end;

                      WriteLn('Deleting rndc.key ...');
                      DeleteFile(LBindPath + CEtcPath + CRndcKey);
                      if not(FileExists(LBindPath + CEtcPath + CRndcKey)) then
                      begin
                        WriteLn('Writing rndc.conf ...');
                        AssignFile(LWriteFile, LBindPath + CEtcPath + CRndcConf);
                        Rewrite(LWriteFile);
                        try
                          for LIndex := 0 to Pred(Length(LKeyFile)) do
                            WriteLn(LWriteFile, LKeyFile[LIndex]);
                          WriteLn(LWriteFile, '');

                          WriteLn(LWriteFile, 'options {');
                          WriteLn(LWriteFile, #$09, 'default-key "', CKeyName, '";');
                          WriteLn(LWriteFile, #$09, 'default-server ', CServerIP, ';');
                          WriteLn(LWriteFile, #$09, 'default-port ', CPortNum, ';');
                          WriteLn(LWriteFile, '};');
                        finally
                          CloseFile(LWriteFile);
                        end;

                        if FileExists(LBindPath + CEtcPath + CRndcConf) then
                        begin
                          WriteLn('Writing named.conf ...');
                          AssignFile(LWriteFile, LBindPath + CEtcPath + CNamedConf);
                          Rewrite(LWriteFile);
                          try
                            WriteLn(LWriteFile, 'options {');
                            WriteLn(LWriteFile, #$09, 'allow-query { ', CServerIP, '; };');
                            WriteLn(LWriteFile, #$09, 'allow-recursion { ', CServerIP, '; };');
                            WriteLn(LWriteFile, #$09, 'directory "', StripBackslash(LBindPath + CEtcPath), '";');
                            WriteLn(LWriteFile, #$09, 'notify no;');
                            WriteLn(LWriteFile, '};');
                            WriteLn(LWriteFile, '');
                            WriteLn(LWriteFile, 'zone "." IN {');
                            WriteLn(LWriteFile, #$09, 'type hint;');
                            WriteLn(LWriteFile, #$09, 'file "', LBindPath, CEtcPath, CNamedRoot, '";');
                            WriteLn(LWriteFile, '};');
                            WriteLn(LWriteFile, '');
                            WriteLn(LWriteFile, 'zone "localhost" IN {');
                            WriteLn(LWriteFile, #$09, 'allow-update { none; };');
                            WriteLn(LWriteFile, #$09, 'file "', LBindPath, CEtcPath, CLocalZone, '";');
                            WriteLn(LWriteFile, #$09, 'type master;');
                            WriteLn(LWriteFile, '};');
                            WriteLn(LWriteFile, '');
                            WriteLn(LWriteFile, 'zone "0.0.127.in-addr.arpa" IN {');
                            WriteLn(LWriteFile, #$09, 'allow-update { none; };');
                            WriteLn(LWriteFile, #$09, 'file "', LBindPath, CEtcPath, CNamedLocal, '";');
                            WriteLn(LWriteFile, #$09, 'type master;');
                            WriteLn(LWriteFile, '};');
                            WriteLn(LWriteFile, '');

                            for LIndex := 0 to Pred(Length(LKeyFile)) do
                              WriteLn(LWriteFile, LKeyFile[LIndex]);
                            WriteLn(LWriteFile, '');

                            WriteLn(LWriteFile, 'controls {');
                            WriteLn(LWriteFile, #$09, 'inet ', CServerIP, ' port ', CPortNum, ' allow { ', CServerIP, '; } keys { "', CKeyName, '"; };');
                            WriteLn(LWriteFile, '};');
                          finally
                            CloseFile(LWriteFile);
                          end;

                          if FileExists(LBindPath + CEtcPath + CNamedConf) then
                          begin
                            WriteLn('Writing localhost.zone ...');
                            AssignFile(LWriteFile, LBindPath + CEtcPath + CLocalZone);
                            Rewrite(LWriteFile);
                            try
                              WriteLn(LWriteFile, '$TTL 3D');
                              WriteLn(LWriteFile, '@', #$09, 'IN', #$09, 'SOA', #$09, 'ns.localhost.', #$09, 'hostmaster.localhost. (');
                              WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '11');
                              WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '8H');
                              WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '2H');
                              WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '4W');
                              WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '1D');
                              WriteLn(LWriteFile, #$09#$09#$09, ')');
                              WriteLn(LWriteFile, '');
                              WriteLn(LWriteFile, #$09, 'IN', #$09, 'NS', #$09, 'ns.localhost.');
                              WriteLn(LWriteFile, '');
                              WriteLn(LWriteFile, 'localhost.', #$09#$09, 'A', #$09, '127.0.0.1');
                              WriteLn(LWriteFile, 'ns.localhost.', #$09#$09, 'A', #$09, '127.0.0.1');
                            finally
                              CloseFile(LWriteFile);
                            end;

                            if FileExists(LBindPath + CEtcPath + CLocalZone) then
                            begin
                              WriteLn('Writing named.local ...');
                              AssignFile(LWriteFile, LBindPath + CEtcPath + CNamedLocal);
                              Rewrite(LWriteFile);
                              try
                                WriteLn(LWriteFile, '$TTL 3D');
                                WriteLn(LWriteFile, '@', #$09, 'IN', #$09, 'SOA', #$09, 'ns.localhost.', #$09, 'hostmaster.localhost. (');
                                WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '11');
                                WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '8H');
                                WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '2H');
                                WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '4W');
                                WriteLn(LWriteFile, #$09#$09#$09#$09#$09, '1D');
                                WriteLn(LWriteFile, #$09#$09#$09, ')');
                                WriteLn(LWriteFile, '');
                                WriteLn(LWriteFile, #$09, 'IN', #$09, 'NS', #$09, 'ns.localhost.');
                                WriteLn(LWriteFile, '');
                                WriteLn(LWriteFile, '1', #$09, 'IN', #$09, 'PTR', #$09, 'localhost.');
                              finally
                                CloseFile(LWriteFile);
                              end;

                              if FileExists(LBindPath + CEtcPath + CNamedLocal) then
                              begin
//                                WriteLn('Reloading configuration ...');
//                                ExecuteProgram(LBindPath + CBinPath + CRndc,
//                                               'reload');

                                Result := not(LStartBind);
                                if not(Result) then
                                begin
                                  WriteLn('Starting service ...');
                                  Result := ServiceStart(CNamed);
                                  if not(Result) then
                                    WriteLn('Service not started!');
                                end;
                              end
                              else
                                WriteLn('named.local not written!');
                            end
                            else
                              WriteLn('localhost.zone not written!');
                          end
                          else
                            WriteLn('named.conf not written!');
                        end
                        else
                          WriteLn('rndc.conf not written!');
                      end
                      else
                        WriteLn('rndc.key not deleted!');
                    end
                    else
                      WriteLn('rndc.key not created!');
                  end
                  else
                    WriteLn('rndc.key not deleted!');
                end
                else
                  WriteLn('named.root not downloaded!');
              end
              else
                WriteLn('named.pid not deleted!');
            end
            else
              WriteLn('Service not stopped!');
          end
          else
            WriteLn('etc-folder not found!');
        end
        else
          WriteLn('rndc-confgen.exe not found!');
//      end
//      else
//        WriteLn('rndc.exe not found!');
    end
    else
      WriteLn('bin-folder not found!');
  end
  else
    WriteLn('Folder not found!');
end;

function DownloadFile(const AURL : String; const AFileName : String) : Boolean;
var
  LAborted      : Boolean;
  LBuffer       : array[0..1023] of Char;
  LBytesRead    : LongWord;
  LBytesWritten : LongInt;
  LNetHandle    : HINTERNET;
  LUrlHandle    : HINTERNET;
  LWriteFile    : File of Char;
begin
  Result := false;

  LNetHandle := InternetOpen('BINDconf', INTERNET_OPEN_TYPE_PRECONFIG, nil, nil, 0);
  if (LNetHandle <> nil) then
  begin
    try
      LUrlHandle := InternetOpenUrl(LNetHandle, PChar(AURL), nil, 0, INTERNET_FLAG_RELOAD, 0);
      if (LUrlHandle <> nil) then
      begin
        try
          LAborted := false;

          AssignFile(LWriteFile, AFileName);
          Rewrite(LWriteFile);
          try
            repeat
              InternetReadFile(LUrlHandle, @LBuffer, SizeOf(LBuffer), LBytesRead);

              if (LBytesRead > 0) then
              begin
                BlockWrite(LWriteFile, LBuffer, LBytesRead, LBytesWritten);
                LAborted := (LBytesRead <> LBytesWritten);
              end;
            until ((LBytesRead = 0) or LAborted);
          finally
            CloseFile(LWriteFile);
          end;

          if not(LAborted) then
            Result := FileExists(AFileName);
        finally
          InternetCloseHandle(LUrlHandle);
        end;
      end;
    finally
      InternetCloseHandle(LNetHandle);
    end;
  end;
end;

function ExecuteProgram(const AFileName : String; const AParameters : String) : Boolean;
var
   LExitCode : LongWord;
   LSEInfo   : TShellExecuteInfo;
begin
  Result := false;

  if FileExists(AFileName) then
  begin
    FillChar(LSEInfo, SizeOf(LSEInfo), 0);
    LSEInfo.cbSize       := SizeOf(TShellExecuteInfo);
    LSEInfo.fMask        := SEE_MASK_NOCLOSEPROCESS;
    LSEInfo.lpDirectory  := PChar(ExtractFilePath(AFileName));
    LSEInfo.lpFile       := PChar(AFileName);
    LSEInfo.lpParameters := PChar(AParameters);
    LSEInfo.nShow        := SW_SHOWNORMAL;
    LSEInfo.Wnd          := 0;

    if ShellExecuteEx(@LSEInfo) then
    begin
      repeat
        Sleep(100);
        
        GetExitCodeProcess(LSEInfo.hProcess, LExitCode);
      until (LExitCode <> STILL_ACTIVE);

      Result := true;
    end;
  end;
end;

function ServiceGetStatus(const AServiceName : String) : LongInt;
var
  LManager : SC_Handle;
  LService : SC_Handle;
  LStatus  : TServiceStatus;
begin
  Result := - 1;

  LManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if (LManager <> 0) then
  begin
    try
      LService := OpenService(LManager, PChar(AServiceName), SERVICE_QUERY_STATUS);
      if (LService <> 0) then
      begin
        try
          if QueryServiceStatus(LService, LStatus) then
            Result := LStatus.dwCurrentState;
        finally
          CloseServiceHandle(LService);
        end;
      end;
    finally
      CloseServiceHandle(LManager);
    end;
  end;
end;

function ServiceStart(const AServiceName : String) : Boolean;
var
  LCheckPoint : LongWord;
  LExit       : Boolean;
  LManager    : SC_Handle;
  LService    : SC_Handle;
  LStatus     : TServiceStatus;
  LTemp       : PChar;
begin
  Result := false;

  if not(Result) then
  begin
    LManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
    if (LManager <> 0) then
    begin
      try
        LService := OpenService(LManager, PChar(AServiceName), SERVICE_START or SERVICE_QUERY_STATUS);
        if (LService <> 0) then
        begin
          try
            LTemp := nil;

            if StartService(LService, 0, LTemp) then
            begin
              if QueryServiceStatus(LService, LStatus) then
              begin
                if (LStatus.dwCurrentState <> SERVICE_RUNNING) then
                begin
                  repeat
                    LCheckPoint := LStatus.dwCheckPoint;
                    Sleep(LStatus.dwWaitHint);

                    LExit := not(QueryServiceStatus(LService, LStatus));
                    if not(LExit) then
                      LExit := (LStatus.dwCheckPoint < LCheckPoint);
                  until ((LStatus.dwCurrentState = SERVICE_RUNNING) or LExit);
                end;

                Result := (LStatus.dwCurrentState = SERVICE_RUNNING);
              end;
            end;
          finally
            CloseServiceHandle(LService);
          end;
        end;
      finally
        CloseServiceHandle(LManager);
      end;
    end;
  end;
end;

function ServiceStop(const AServiceName : String) : Boolean;
var
  LCheckPoint : LongWord;
  LExit       : Boolean;
  LManager    : SC_Handle;
  LService    : SC_Handle;
  LStatus     : TServiceStatus;
begin
  Result := false;

  if not(Result) then
  begin
    LManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
    if (LManager <> 0) then
    begin
      try
        LService := OpenService(LManager, PChar(AServiceName), SERVICE_STOP or SERVICE_QUERY_STATUS);
        if (LService <> 0) then
        begin
          try
            if ControlService(LService, SERVICE_CONTROL_STOP, LStatus) then
            begin
              if QueryServiceStatus(LService, LStatus) then
              begin
                if (LStatus.dwCurrentState <> SERVICE_STOPPED) then
                begin
                  repeat
                    LCheckPoint := LStatus.dwCheckPoint;
                    Sleep(LStatus.dwWaitHint);

                    LExit := not(QueryServiceStatus(LService, LStatus));
                    if not(LExit) then
                      LExit := (LStatus.dwCheckPoint < LCheckPoint);
                  until ((LStatus.dwCurrentState = SERVICE_STOPPED) or LExit);
                end;

                Result := (LStatus.dwCurrentState = SERVICE_STOPPED);
              end;
            end;
          finally
            CloseServiceHandle(LService);
          end;
        end;
      finally
        CloseServiceHandle(LManager);
      end;
    end;
  end;
end;

function StripBackslash(const APath : String) : String;
const
  CBackslash = '\';
begin
  Result := APath;
  if (Result[Length(Result)] = CBackSlash) then
    Delete(Result, Length(Result), 1);
end;

begin
  WriteLn('BINDconf 0.2b1');
  WriteLn('(C) 2009 coltishWARE');
  WriteLn('');

  if ConfigureBIND then
  begin
    WriteLn('');
    WriteLn('CONFIGURATION SUCCEEDED');
  end
  else
  begin
    WriteLn('');
    WriteLn('CONFIGURATION FAILED');
  end;
end.
